MockNwlistopdatOp1 = [
    " 1\n",
    "FORWARD  (ARQ. DE DADOS)    : forward.dat\n",
    "FORWARDH (ARQ. CABECALHOS)  : forwarh.dat\n",
    "NEWDESP  (REL. CONFIGS)     : newdesp.dat\n",
    "-----------------------------------------\n",
    " XXX XXX    PERIODOS INICIAL E FINAL\n",
    "   1   1\n",
    " XXXX XXXX  SERIE INICIAL E FINAL\n",
    "    1    1\n",
]

MockNwlistopdatOp2 = [
    " 2\n",
    "FORWARD  (ARQ. DE DADOS)    : forward.dat\n",
    "FORWARDH (ARQ. CABECALHOS)  : forwarh.dat\n",
    "NEWDESP  (REL. CONFIGS)     : newdesp.dat\n",
    "-----------------------------------------\n",
    " XXX XXX    PERIODOS INICIAL E FINAL\n",
    "   1  58\n",
    " 1-CMO           2-DEFICITS         3-ENA CONTROL.   4-EARM FINAL       5-ENA FIO BRUTA 6-EVAPORACAO    7-VERTIMENTO     38-SOMA AFL.PAS.                  43-VIOL. DA REST. LPP DEFL. MAX.       48-VOLUME BOMBEADO EST. BOMB.     53-VIOLACAO RHV\n",
    " 8-VAZAO MIN.    9-GER.HIDR.CONT   10-GER. TERMICA  11-INTERCAMBIOS    12-MERC.LIQ.    13-VALOR AGUA   14-VOLUME MORTO   39-GER. EOLICA                    44-RHS DA REST. LPP TURB. MAX.         49-CONSUMO EST. BOMB.             54-VALOR FORMULA RHV\n",
    "15-EXCESSO      16-GHMAX           17-OUTROS USOS   18-BENEF.INT/AGR   19-F.CORR.EC    20-GHTOTAL      21-ENA BRUTA      40-VEL. DE VENTO                  45-RHS DA REST. LPP DEFL. MAX.         50-VIOLACAO RHQ                   55-CUSTO VIOLACAO RHV\n",
    "22-ACOPLAMENTO  23-INVASAO CG      24-PENAL.INV.CG. 25-ACIONAMENTO MAR 26-COPER        27-CTERM        28-CDEFICIT       41-VIOL. DA REST. FTE             46-VIOL. DAS REST. ELETRI ESP.         51-VALOR FORMULA RHQ\n",
    "29-GER.FIO LIQ. 30-PERDA FIO       31-ENA FIO LIQ.  32-BENEF. GNL      33-VIOL.GHMIN   34-PERDAS       37-GEE            42-VIOL. DA REST. LPP TURB. MAX.  47-CUSTO VIOL. DAS REST. ELETRI ESP.   52-CUSTO VIOLACAO RHQ\n",
    " XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX (SE 99 CONSIDERA TODAS)\n",
    " 99\n",
    "-----------------------------------------------------------------------------------------------------------------------\n",
    " 1-VOL.ARMAZ       2-GER.HID         3-VOL.TURB.     4-VOL. VERT.      5-VIOL.GHMIN    6-ENCH.MORTO   7-VIOL. DEPMIN. 15 - SOMA AFL. LAG PAR(p)-A  17 - VIOL. TURB. MAX.   19 - VIOL. DA REST. LPP TURB. MAX.   21 - RHS DA REST. LPP TURB. MAX.    23 - VOLUME DESVIADO DA USINA\n",
    " 8-DESV. AGUA      9-DESV. POS.      10-DESVIO NEG.  11-VIOL. FPGHA   12-VAZAO AFL.  13-VAZAO INCREM. 14-VARM PCT.    16 - VIOL. DEFL. MAX.        18 - VIOL. TURB. MIN.   20 - VIOL. DA REST. LPP DEFL. MAX.   22 - RHS DA REST. LPP DEFL. MAX. \n",
    " XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX (SE 99 CONSIDERA TODAS)\n",
    " 99\n",
    " XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX  (SE 999 CONSIDERA TODAS AS USINAS)\n",
    " 999 \n",
]

MockNwlistopdatOp4 = [
    " 4\n",
    "FORWARD  (ARQ. DE DADOS)    : forward.dat\n",
    "FORWARDH (ARQ. CABECALHOS)  : forwarh.dat\n",
    "NEWDESP  (REL. CONFIGS)     : newdesp.dat\n",
    "-----------------------------------------\n",
    " XXX XXX    PERIODOS INICIAL E FINAL\n",
    "   1  58\n",
    " 1-CMO           2-DEFICITS         3-ENA CONTROL.   4-EARM FINAL       5-ENA FIO BRUTA 6-EVAPORACAO    7-VERTIMENTO     38-SOMA AFL.PAS.                  43-VIOL. DA REST. LPP DEFL. MAX.       48-VOLUME BOMBEADO EST. BOMB.     53-VIOLACAO RHV\n",
    " 8-VAZAO MIN.    9-GER.HIDR.CONT   10-GER. TERMICA  11-INTERCAMBIOS    12-MERC.LIQ.    13-VALOR AGUA   14-VOLUME MORTO   39-GER. EOLICA                    44-RHS DA REST. LPP TURB. MAX.         49-CONSUMO EST. BOMB.             54-VALOR FORMULA RHV\n",
    "15-EXCESSO      16-GHMAX           17-OUTROS USOS   18-BENEF.INT/AGR   19-F.CORR.EC    20-GHTOTAL      21-ENA BRUTA      40-VEL. DE VENTO                  45-RHS DA REST. LPP DEFL. MAX.         50-VIOLACAO RHQ                   55-CUSTO VIOLACAO RHV\n",
    "22-ACOPLAMENTO  23-INVASAO CG      24-PENAL.INV.CG. 25-ACIONAMENTO MAR 26-COPER        27-CTERM        28-CDEFICIT       41-VIOL. DA REST. FTE             46-VIOL. DAS REST. ELETRI ESP.         51-VALOR FORMULA RHQ\n",
    "29-GER.FIO LIQ. 30-PERDA FIO       31-ENA FIO LIQ.  32-BENEF. GNL      33-VIOL.GHMIN   34-PERDAS       37-GEE            42-VIOL. DA REST. LPP TURB. MAX.  47-CUSTO VIOL. DAS REST. ELETRI ESP.   52-CUSTO VIOLACAO RHQ\n",
    " XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX (SE 99 CONSIDERA TODAS)\n",
    " 99\n",
    "-----------------------------------------------------------------------------------------------------------------------\n",
    " 1-VOL.ARMAZ       2-GER.HID         3-VOL.TURB.     4-VOL. VERT.      5-VIOL.GHMIN    6-ENCH.MORTO   7-VIOL. DEPMIN. 15 - SOMA AFL. LAG PAR(p)-A  17 - VIOL. TURB. MAX.   19 - VIOL. DA REST. LPP TURB. MAX.   21 - RHS DA REST. LPP TURB. MAX.    23 - VOLUME DESVIADO DA USINA\n",
    " 8-DESV. AGUA      9-DESV. POS.      10-DESVIO NEG.  11-VIOL. FPGHA   12-VAZAO AFL.  13-VAZAO INCREM. 14-VARM PCT.    16 - VIOL. DEFL. MAX.        18 - VIOL. TURB. MIN.   20 - VIOL. DA REST. LPP DEFL. MAX.   22 - RHS DA REST. LPP DEFL. MAX. \n",
    " XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX XX (SE 99 CONSIDERA TODAS)\n",
    " 99\n",
    " XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX  (SE 999 CONSIDERA TODAS AS USINAS)\n",
    " 999 \n",
]
